@@ -290,43 +290,116 @@ return ( );
 }
 
 # local_disk_space([&always-count])
-# Returns the total local and free disk space on the system.
+# Returns the total local and free disk space on the system, plus a list of
+# per-filesystem total and free
 sub local_disk_space
 {
 my ($always) = @_;
-my ($total, $free) = (0, 0);
+my ($total, $free, $used) = (0, 0, 0);
+my @fs;
 my @mounted = &mount::list_mounted();
 my %donezone;
 my %donevzfs;
 my %donedevice;
-foreach $m (@mounted) {
+my %donedevno;
+
+# Get list of zone pools
+my %zpools = ( 'zones' => 1, 'zroot' => 1 );
+if (&has_command("zpool")) {
+	foreach my $flag ("-P", "-p") {
+		my @out = &backquote_command("zpool list $flag 2>/dev/null");
+		foreach my $l (@out) {
+			if (/^(\S+)\s+(\d+)\s+(\d+)\s+(\d+)/) {
+				$zpools{$1} = [ $2 / 1024, $4 / 1024 ];
+				}
+			}
+		}
+	}
+
+# Add up all local filesystems
+foreach my $m (@mounted) {
 	if ($m->[2] =~ /^ext/ ||
-	    $m->[2] eq "reiserfs" || $m->[2] eq "ufs" ||
+	    $m->[2] eq "reiserfs" || $m->[2] eq "ufs" || $m->[2] eq "f2fs" ||
 	    $m->[2] eq "zfs" || $m->[2] eq "simfs" || $m->[2] eq "vzfs" ||
-	    $m->[2] eq "xfs" || $m->[2] eq "jfs" ||
+	    $m->[2] eq "xfs" || $m->[2] eq "jfs" || $m->[2] eq "btrfs" ||
+	    $m->[2] eq "apfs" || $m->[2] eq "fuseblk" ||
 	    $m->[1] =~ /^\/dev\// ||
 	    &indexof($m->[1], @$always) >= 0) {
-		if ($m->[1] =~ /^zones\/([^\/]+)/ &&
-                    $m->[2] eq "zfs" && $donezone{$1}++) {
-			# Don't double-count zones
+		my $zp;
+		if ($m->[1] =~ /^([^\/]+)(\/(\S+))?/ &&
+                    $m->[2] eq "zfs" && $zpools{$1}) {
+			# Don't double-count maps from the same zone pool
+			next if ($donezone{$1}++);
+			$zp = $zpools{$1};
+			}
+		if ($donedevice{$m->[0]}++ ||
+		    $donedevice{$m->[1]}++) {
+			# Don't double-count mounts from the same device, or
+			# on the same directory.
 			next;
 			}
-		if ($donedevice{$m->[0]}++) {
-			# Don't double-count mounts from the same device
+		my @st = stat($m->[0]);
+		if (@st && $donedevno{$st[0]}++) {
+			# Don't double-count same filesystem by device number
 			next;
 			}
-		my ($t, $f) = &mount::disk_space($m->[2], $m->[0]);
+		if ($m->[1] eq "/dev/fuse") {
+			# Skip fuse user-space filesystem mounts
+			next;
+			}
+		if ($m->[2] eq "swap") {
+			# Skip virtual memory
+			next;
+			}
+		if ($m->[2] eq "squashfs") {
+			# Skip /snap mounts
+			next;
+			}
+		if ($m->[1] =~ /^\/dev\/sr/) {
+			# Skip CDs
+			next;
+			}
+		# apfs count only physical devices not containers shares
+		if ($m->[2] eq "apfs" && $m->[3] =~ /nobrowse/) {
+			next;
+			}
+		# Get the size - for ZFS mounts, this comes from the underlying
+		# total pool size and free
+		my ($t, $f);
+		if ($zp) {
+			($t, $f) = @$zp;
+			}
+		else {
+			($t, $f, $u, $p) = &disk_space($m->[2], $m->[0]);
+			}
 		if (($m->[2] eq "simfs" || $m->[2] eq "vzfs" ||
-		     $m->[0] eq "/dev/vzfs") &&
+		     $m->[0] eq "/dev/vzfs" ||
+		     $m->[0] eq "/dev/simfs") &&
 		    $donevzfs{$t,$f}++) {
 			# Don't double-count VPS filesystems
 			next;
 			}
 		$total += $t*1024;
 		$free += $f*1024;
+		$used += $u*1024;
+		my ($it, $if);
+		if (defined(&inode_space)) {
+			($it, $if, $iu, $ip) = &inode_space($m->[2], $m->[0]);
+			}
+		push(@fs, { 'total' => $t*1024,
+			    'free' => $f*1024,
+			    'used' => $u*1024,
+			    'used_percent' => $p,
+			    'itotal' => $it,
+			    'ifree' => $if,
+			    'iused' => $iu,
+			    'iused_percent' => $ip,
+			    'dir' => $m->[0],
+			    'device' => $m->[1],
+			    'type' => $m->[2] });
 		}
 	}
-return ($total, $free);
+return ($total, $free, \@fs, $used);
 }
 
 1;
